package com.afp.ordermanagement.service;

import com.afp.ordermanagement.controller.AllOrdersRequestBody;
import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.model.OrderStatus;
import com.afp.ordermanagement.repository.OrderRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;

import org.springframework.data.mongodb.core.ReactiveMongoTemplate;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.List;

import static java.util.Objects.isNull;

@Service
public class OrderService {

    @Autowired
    OrderRepository orderRepository;

    @Autowired
    ReactiveMongoTemplate reactiveMongoTemplate;

    public Mono<Order> createOrder(Order newOrder){
        System.out.println("here");
        String defaultOrderTrackingCode = "N/A";
        OrderStatus defaultOrderStatus = OrderStatus.RECEIVED;
        long serviceSystemTime = System.currentTimeMillis();
        newOrder.setOrderStatus(defaultOrderStatus);
        newOrder.setOrderTrackingCode(defaultOrderTrackingCode);
        newOrder.setOrderCreatedAt(serviceSystemTime);
        newOrder.setOrderUpdatedAt(serviceSystemTime);
        //newOrder.setId("abc123");
        //System.out.println(newOrder);
        return orderRepository.save(newOrder);
    }

    public Flux<Order> getAllOrders(){
        return orderRepository.findAll();
    }

    public Flux<Order> getQueriedOrders(AllOrdersRequestBody requestBody){
        Query query = new Query();
        // ---- pagination & sort ----
        int page = requestBody.pagination.page, size = requestBody.pagination.size;
        Sort sortByLastUpdated = Sort.by("orderUpdatedAt").descending();
        Pageable pagination = PageRequest.of(page, size, sortByLastUpdated);
        query.with(pagination);
        // ---- date ----
        long startDate = requestBody.date.start, endDate = requestBody.date.end;
        AllOrdersRequestBody.DateFilter dateFilter = requestBody.date.filter;
        if (startDate > 0 && endDate > 0 && startDate <= endDate) {
            if (!isNull(dateFilter)) {
                query.addCriteria(Criteria.where(dateFilter.toString()).gte(startDate).lte(endDate));
            } else {
                Criteria dateCriteria = Criteria.where("orderCreatedAt").gte(startDate).lte(endDate)
                        .orOperator(Criteria.where("orderUpdatedAt").gte(startDate).lte(endDate));
                query.addCriteria(dateCriteria);
            }
        }
        // ---- search ----
        String queryText = requestBody.search.query;
        if (!isNull(queryText) && !queryText.trim().isEmpty()) {
            String searchByStr = requestBody.search.by.toString();
            Criteria searchCriteria = Criteria.where(searchByStr).regex("^" + queryText);
            query.addCriteria(searchCriteria);
        }
        // ---- status ----
        List<OrderStatus> truthyStatuses = requestBody.status.getTruthyStatuses();
        if (!truthyStatuses.isEmpty()) {
            Criteria statusCriteria = Criteria.where("orderStatus").in(truthyStatuses);
            query.addCriteria(statusCriteria);
        }

        return reactiveMongoTemplate.find(query, Order.class);
    }


    public Mono<Order> getOrderById(String orderId) {
        return orderRepository.findById(orderId);
    }

    public Flux<Order> getAllOrdersByCustomerId(String customerId){
        return orderRepository.findByCustomerId(customerId);
    }

    public Mono<Order> updateOrderByOrderId(String orderId, Order newOrder){

        return orderRepository.findById(orderId)
                .flatMap(existingOrder -> {
                    //existingOrder.setCustomerAddress(newOrder.getCustomerAddress());
                    //existingOrder.setCustomerEmailAddress(newOrder.getCustomerEmailAddress());
                    //existingOrder.setOrderTrackingCode(newOrder.getOrderTrackingCode());
                    //existingOrder.setOrderItems(newOrder.getOrderItems());
                    existingOrder.setOrderStatus(newOrder.getOrderStatus());
                    //existingOrder.setOrderItems(newOrder.getOrderItems());
                    System.out.println("serviceeee" + existingOrder);
                    return orderRepository.save(existingOrder);
                });
    }

    public void deleteOrderById(String orderId) {
        orderRepository.deleteById(orderId);
    }
}
